
import java.io.UnsupportedEncodingException;
import java.security.NoSuchAlgorithmException;
import java.util.*;
import java.security.MessageDigest;

/**
 * 签名计算工具类
 */
public class SignHelper {

    /**
     * 加密方法入口
     */
    public static String generateSign(Map<String, String> params, String secret) throws NoSuchAlgorithmException, UnsupportedEncodingException {
        String encodeString = getEncodeString(params, secret);
        System.out.println("encodeString:" + encodeString);
        String sign = generateSign(encodeString);
        System.out.println("sign:" + sign);
        return sign;
    }

    private static String getEncodeString(Map<String, String> params, String secret) {
        Iterator<String> keyIter = params.keySet().iterator();
        Set<String> sortedParams = new TreeSet<>();
        while (keyIter.hasNext()) {
            sortedParams.add(keyIter.next());
        }

        StringBuilder strB = new StringBuilder(secret);

        // 排除sign和空值参数
        for (String key : sortedParams) {
            if (key.equals("sign")) {
                continue;
            }

            String value = params.get(key);

            if (value != null && !"".equals(value)) {
                strB.append(key).append(value);
            }
        }
        return strB.toString();
    }

    private static String generateSign(String content) throws NoSuchAlgorithmException, UnsupportedEncodingException {
        return Sha1(content).toLowerCase();
    }

 /**
     * SHA1 加密
     *
     * @param s 待加密的字符串
     * @return
     */
    private final static String Sha1(String s) throws NoSuchAlgorithmException, UnsupportedEncodingException {
        char hexDigits[] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'};
        try {
            byte[] btInput = s.getBytes("utf-8");
            // 获得MD5摘要算法的 MessageDigest 对象
            MessageDigest mdInst = MessageDigest.getInstance("sha-1");
            // 使用指定的字节更新摘要
            mdInst.update(btInput);
            // 获得密文
            byte[] md = mdInst.digest();
            // 把密文转换成十六进制的字符串形式
            int j = md.length;
            char str[] = new char[j * 2];
            int k = 0;
            for (int i = 0; i < j; i++) {
                byte byte0 = md[i];
                str[k++] = hexDigits[byte0 >>> 4 & 0xf];
                str[k++] = hexDigits[byte0 & 0xf];
            }
            return new String(str);
        } catch (Exception e) {
            System.out.println("Sha1 error");
            throw e;
        }
    }

    public static void main(String[] args) throws UnsupportedEncodingException, NoSuchAlgorithmException {
        String secret = ".!>UIra?Q45643N!}Hs>OIIrFcfoV43R6J,[Z.Lzvd@P0O!yHRI-o-/!0f=`}Q!w";
        Map<String, String> params = new HashMap<>();
        params.put("developerId","012407a2f37644a2b74610fc71cc049a");
        params.put("timestamp","1650620563");
        params.put("version","v1.0");
        params.put("test","");
        params.put("orderId","327389290032994");
        SignHelper.generateSign(params,secret);
    }

}